<?php
// $Id: 10_PHP.php.create.Vsite
// Copyright (c) 2008-2023 Michael Stauber, SOLARSPEED.NET
// Copyright (c) 2008-2023 Team BlueOnyx, BLUEONYX.IT
//
// ui extension to set settings for php Vsite creation

if(defined("PHP_EXTENSION_CREATE_VSITE"))
{
    return 1;
}
define("PHP_EXTENSION_CREATE_VSITE", 1);

include_once("UIExtension.php");

class PHPExtension extends UIExtension
{
    function display(&$helper, &$container, &$cce_info)
    {
        $CI = get_instance();
        $BX_SESSION = $CI->getBX_SESSION();
        $factory = $helper->getHtmlComponentFactory("base-apache");
        $cce = $helper->getCceClient();
    
        $defaults = $cce->get($cce_info["CCE_SERVICES_OID"], "PHP");
        
        // only add page if not already there
        if ($cce_info['CAN_ADD_PAGE'])
        {
            $pageId = 'webScripting';
            if (!in_array('webScripting', $container->getPageIds())) {
                $container->addPage("webScripting", $factory->getLabel("webScripting"));
            }
        }
        else {
            $pageId = $cce_info['PAGED_BLOCK_DEFAULT_PAGE'];
        }

        // Known PHP versions:
        $known_php_versions = array(
                                'PHP56' => '5.6',
                                'PHP70' => '7.0',
                                'PHP71' => '7.1',
                                'PHP72' => '7.2',
                                'PHP73' => '7.3',
                                'PHP74' => '7.4',
                                'PHP80' => '8.0',
                                'PHP81' => '8.1',
                                'PHP82' => '8.2',
                                'PHP83' => '8.3',
                                'PHP84' => '8.4',
                                'PHP85' => '8.5',
                                'PHP86' => '8.6',
                                'PHP90' => '9.0',
                                'PHP91' => '9.1',
                                'PHP92' => '9.2',
                                'PHP93' => '9.3',
                                'PHP94' => '9.4',
                                );

        // Get main "PHP" Object:
        $Master_PHP = $cce->getObject("PHP");

        $all_php_versions = array('PHPOS' => $Master_PHP['PHP_version_os']);

        foreach ($known_php_versions as $NSkey => $NSvalue) {
            $extraPHPs[$NSkey] = $cce->get($Master_PHP["OID"], $NSkey);
            if ($extraPHPs[$NSkey]['present'] != "1") {
                unset($extraPHPs[$NSkey]);
            }
        }

        $all_selectable_php_versions['PHPOS'] = $Master_PHP['PHP_version_os'];
        foreach ($extraPHPs as $NSkey => $NSvalue) {
            if ($NSvalue['present'] == '1') {
                $all_php_versions[$NSvalue['NAMESPACE']] = $NSvalue['version'];
                $all_php_versions_reverse[$NSvalue['version']] = $NSvalue['NAMESPACE'];
                if ($NSvalue['enabled'] == '1') {
                    $all_selectable_php_versions[$NSvalue['NAMESPACE']] = $NSvalue['version'];
                }
            }
        }

        // $defaults = Array
        // (
        //     [NAMESPACE] => PHP
        //     [CLASSVER] => 1.0
        //     [suPHP_enabled] => 0
        //     [mod_ruid_enabled] => 0          <-- DSO + mod_ruid2 - Deprecated in 5211R and onward
        //     [fpm_enabled] => 0
        //     [version] => 
        //     [enabled] => 0                   <-- Regular DSO PHP - Deprecated in 5211R and onward
        //     [featureEnabled] => 1
        //     [i18nName] => [[base-apache.phpI18nName]]
        // )

        if (($defaults['enabled'] == "1") && ($defaults['mod_ruid_enabled'] == "0") && ($defaults['suPHP_enabled'] == "0") && ($defaults['fpm_enabled'] == "0")) {
            // This would normally enable PHP as DSO, but in 5211R (and onward) this is disabled:
            //$php_enabler = array("disabled" => "0", "php" => "1", "enableModRUID" => "0", "enableSuPHP" => "0", "enableModFPM" => "0");
            // As a fallback we switch this to PHP-FPM in case it is still set:
            $php_enabler = array("disabled" => "0", "enableSuPHP" => "0", "enableModFPM" => "1");

        }
        elseif (($defaults['enabled'] == "1") && ($defaults['mod_ruid_enabled'] == "0") && ($defaults['suPHP_enabled'] == "1") && ($defaults['fpm_enabled'] == "0")) { 
            // This enables suPHP:
            $php_enabler = array("disabled" => "0", "enableSuPHP" => "1", "enableModFPM" => "0"); 
        }
        elseif (($defaults['enabled'] == "1") && ($defaults['mod_ruid_enabled'] == "1") && ($defaults['suPHP_enabled'] == "0") && ($defaults['fpm_enabled'] == "0")) { 
            // This would normally enable PHP as DSO+mod_ruid2, but in 5211R (and onward) this is disabled:
            //$php_enabler = array("disabled" => "0", "php" => "0", "enableModRUID" => "1", "enableSuPHP" => "0", "enableModFPM" => "0");
            // As a fallback we switch this to PHP-FPM in case it is still set:
            $php_enabler = array("disabled" => "0", "enableSuPHP" => "0", "enableModFPM" => "1"); 
        }
        elseif (($defaults['enabled'] == "1") && ($defaults['mod_ruid_enabled'] == "0") && ($defaults['suPHP_enabled'] == "0") && ($defaults['fpm_enabled'] == "1")) { 
            // This enables PHP-FPM:
            $php_enabler = array("disabled" => "0", "enableSuPHP" => "0", "enableModFPM" => "1"); 
        }
        else {
            // Fallback: PHP is off entirely:
            $php_enabler = array("disabled" => "1", "enableSuPHP" => "0", "enableModFPM" => "0"); 
        }

        //
        //-- Reseller: Can the reseller that owns this Vsite modify this?
        //
        $VsiteOwnerObj = $BX_SESSION['loginUser'];
        if ($VsiteOwnerObj['name'] != "admin") {
            $resellerCaps = $cce->scalar_to_array($VsiteOwnerObj['capabilities']);
            if (!in_array('resellerPHP', $resellerCaps)) {
                unset($php_enabler['php']);
            }
            if (!in_array('resellerRUID', $resellerCaps)) {
                unset($php_enabler['enableModRUID']);
            }
            if (!in_array('resellerSUPHP', $resellerCaps)) {
                unset($php_enabler['enableSuPHP']);
            }
            if (!in_array('resellerFPM', $resellerCaps)) {
                unset($php_enabler['enableModFPM']);
            }
        }

        // Walk through the array and make sure that at least one option is set:
        foreach ($php_enabler as $key => $value) {
            if (($key = array_search('1', $php_enabler)) === false) {
                $php_enabler['disabled'] = "1";
            }
        }
        //-- 

        $xxx = $factory->getRadio("PHP_enabled", $php_enabler, "rw");
        $container->addFormField(
                $xxx,
                $factory->getLabel("PHP_enabled"),
                $pageId
                );

        $AKPE = array_keys($php_enabler);

        if (
            (($defaults['fpm_enabled'] == "1") || ($defaults['suPHP_enabled'] == "1")) || 
            ((!in_array('enabled', $AKPE)) && (!in_array('enableModRUID', $AKPE)) && ((in_array('enableSuPHP', $AKPE)) || (in_array('enableModFPM', $AKPE))))
            ) {
            $usedPHPversion = "";

            // Add a pulldown that allows to change the default PHP version of Apache to
            // one of the extra PHP versions:
            $PHP_version_Field_select = $factory->getMultiChoice("PHP_version_Field", array_values($all_selectable_php_versions));
            if (!isset($defaults['version'])) {
                $defaults['version'] = 'PHPOS';
            }
            if (isset($defaults['version'])) {
                if (($defaults['version'] == '') || (!preg_match('/^PHP(.*)/', $defaults['version']))) {
                    $defaults['version'] = 'PHPOS';
                }
            }
            $PHP_version_Field_select->setSelected($all_php_versions[$defaults['version']], true);
            $container->addFormField($PHP_version_Field_select, $factory->getLabel("PHP_version_Field"), $pageId);

        }
        else {
            $usedPHPversion = $Master_PHP['PHP_version'];
            // PHP_version being used by this Vsite:
            $PHP_version_Field = $factory->getTextField("PHP_version", $usedPHPversion, "r");
            $PHP_version_Field->setOptional ('silent');
            $container->addFormField(
                $PHP_version_Field,
                $factory->getLabel("PHP_version"),
                $pageId
            );
        }

    }

    function handle(&$helper, &$errors, &$cce_info, &$attributes)
    {
        $this->attributes = $attributes;

        $cce = $helper->getCceClient();

        // Known PHP versions:
        $known_php_versions = array(
                                'PHP56' => '5.6',
                                'PHP70' => '7.0',
                                'PHP71' => '7.1',
                                'PHP72' => '7.2',
                                'PHP73' => '7.3',
                                'PHP74' => '7.4',
                                'PHP80' => '8.0',
                                'PHP81' => '8.1',
                                'PHP82' => '8.2',
                                'PHP83' => '8.3',
                                'PHP84' => '8.4',
                                'PHP85' => '8.5',
                                'PHP86' => '8.6',
                                'PHP90' => '9.0',
                                'PHP91' => '9.1',
                                'PHP92' => '9.2',
                                'PHP93' => '9.3',
                                'PHP94' => '9.4',
                                );

        // Get main "PHP" Object:
        $Master_PHP = $cce->getObject("PHP");

        $all_php_versions = array('PHPOS' => $Master_PHP['PHP_version_os']);

        foreach ($known_php_versions as $NSkey => $NSvalue) {
            $extraPHPs[$NSkey] = $cce->get($Master_PHP["OID"], $NSkey);
            if ($extraPHPs[$NSkey]['present'] != "1") {
                unset($extraPHPs[$NSkey]);
            }
        }

        $all_selectable_php_versions['PHPOS'] = $Master_PHP['PHP_version_os'];
        $all_php_versions_reverse[$Master_PHP['PHP_version_os']] = 'PHPOS';
        foreach ($extraPHPs as $NSkey => $NSvalue) {
            if ($NSvalue['present'] == '1') {
                $all_php_versions[$NSvalue['NAMESPACE']] = $NSvalue['version'];
                $all_php_versions_reverse[$NSvalue['version']] = $NSvalue['NAMESPACE'];
                if ($NSvalue['enabled'] == '1') {
                    $all_selectable_php_versions[$NSvalue['NAMESPACE']] = $NSvalue['version'];
                }
            }
        }

        // Note the double check. On additional form submit after an error radio settings
        // can change from "value" to "&value&" due to serialization. So we check for both:

        if (($this->attributes['PHP_enabled'] == "enableSuPHP") || ($this->attributes['PHP_enabled'] == "&enableSuPHP&")) {
          $php_params = array( 
                        'enabled' => "1", 
                        'mod_ruid_enabled' => "0",
                        'suPHP_enabled' => "1",
                        'fpm_enabled' => "0"
                    );
        }
        elseif (($this->attributes['PHP_enabled'] == "enableModFPM") || ($this->attributes['PHP_enabled'] == "&enableModFPM&")) {
          $php_params = array( 
                        'enabled' => "1", 
                        'mod_ruid_enabled' => "0",
                        'suPHP_enabled' => "0",
                        'fpm_enabled' => "1"
                    );
        }
        else {
          $php_params = array( 
                        'enabled' => "0", 
                        'mod_ruid_enabled' => "0",
                        'suPHP_enabled' => "0",
                        'fpm_enabled' => "0"
                    );
        }

        // Handle change of PHP version:
        if (isset($this->attributes['PHP_version_Field'])) {
            // Result from pulldown. We are allowed to change the version.
            $php_params['version'] = $all_php_versions_reverse[$this->attributes['PHP_version_Field']];
            
        }
        if (isset($this->attributes['PHP_version'])) {
            // Result from TextField. We are NOT allowed to change the version and use
            // the one that Apache is using:
            if ($Master_PHP['PHP_version'] == '') {
                $Master_PHP['PHP_version'] = 'PHPOS';
            }
            if (isset($all_php_versions_reverse[$Master_PHP['PHP_version']])) {
                $php_params['version'] = $all_php_versions_reverse[$Master_PHP['PHP_version']];
            }
            else {
                $php_params['version'] = $Master_PHP['PHP_version'];
            }
        }

        if (($php_params['fpm_enabled'] == "0") && ($php_params['suPHP_enabled'] == "0")) {
            if (isset($all_php_versions_reverse[$Master_PHP['PHP_version']])) {
                $php_params['version'] = $all_php_versions_reverse[$Master_PHP['PHP_version']];
            }
            else {
                $php_params['version'] = $Master_PHP['PHP_version'];
            }
        }

        $cce->set($cce_info["CCE_OID"], 'PHP', $php_params); 

        if (($php_params['suPHP_enabled'] == "1") || ($php_params['fpm_enabled'] == "1")) {
            // Disable UserWebs if suPHP or PHP-FPM is activated
            $cce->set($cce_info["CCE_OID"], 'USERWEBS', array('enabled' => "0"));
        }

        $errors = array_merge($errors, $cce->errors());
    }
}
?>